@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,19 +16,26 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api_core import operation as gac_operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
 from google.cloud.firestore_admin_v1.services.firestore_admin import pagers
+from google.cloud.firestore_admin_v1.types import database
+from google.cloud.firestore_admin_v1.types import database as gfa_database
 from google.cloud.firestore_admin_v1.types import field
 from google.cloud.firestore_admin_v1.types import field as gfa_field
 from google.cloud.firestore_admin_v1.types import firestore_admin
@@ -36,13 +43,43 @@ from google.cloud.firestore_admin_v1.types import index
 from google.cloud.firestore_admin_v1.types import index as gfa_index
 from google.cloud.firestore_admin_v1.types import operation as gfa_operation
 from google.protobuf import empty_pb2  # type: ignore
+from google.protobuf import field_mask_pb2  # type: ignore
 from .transports.base import FirestoreAdminTransport, DEFAULT_CLIENT_INFO
 from .transports.grpc_asyncio import FirestoreAdminGrpcAsyncIOTransport
 from .client import FirestoreAdminClient
 
 
 class FirestoreAdminAsyncClient:
-    """Operations are created by service ``FirestoreAdmin``, but are
+    """The Cloud Firestore Admin API.
+
+    This API provides several administrative services for Cloud
+    Firestore.
+
+    Project, Database, Namespace, Collection, Collection Group, and
+    Document are used as defined in the Google Cloud Firestore API.
+
+    Operation: An Operation represents work being performed in the
+    background.
+
+    The index service manages Cloud Firestore indexes.
+
+    Index creation is performed asynchronously. An Operation resource is
+    created for each such asynchronous operation. The state of the
+    operation (including any errors encountered) may be queried via the
+    Operation resource.
+
+    The Operations collection provides a record of actions performed for
+    the specified Project (including any Operations in progress).
+    Operations are not created directly but through calls on other
+    collections or resources.
+
+    An Operation that is done may be deleted so that it is no longer
+    listed as part of the Operation collection. Operations are garbage
+    collected after 30 days. By default, ListOperations will only return
+    in progress and failed operations. To list completed operation,
+    issue a ListOperations request with the filter ``done: true``.
+
+    Operations are created by service ``FirestoreAdmin``, but are
     accessed via service ``google.longrunning.Operations``.
     """
 
@@ -119,6 +156,42 @@ class FirestoreAdminAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return FirestoreAdminClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> FirestoreAdminTransport:
         """Returns the transport used by the client instance.
@@ -181,11 +254,11 @@ class FirestoreAdminAsyncClient:
 
     async def create_index(
         self,
-        request: firestore_admin.CreateIndexRequest = None,
+        request: Union[firestore_admin.CreateIndexRequest, dict] = None,
         *,
         parent: str = None,
         index: gfa_index.Index = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -195,8 +268,32 @@ class FirestoreAdminAsyncClient:
         metadata for the operation will be the type
         [IndexOperationMetadata][google.firestore.admin.v1.IndexOperationMetadata].
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_create_index():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.CreateIndexRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                operation = client.create_index(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.CreateIndexRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.CreateIndexRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.CreateIndex][google.firestore.admin.v1.FirestoreAdmin.CreateIndex].
             parent (:class:`str`):
@@ -228,7 +325,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, index])
         if request is not None and has_flattened_params:
@@ -276,17 +373,37 @@ class FirestoreAdminAsyncClient:
 
     async def list_indexes(
         self,
-        request: firestore_admin.ListIndexesRequest = None,
+        request: Union[firestore_admin.ListIndexesRequest, dict] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListIndexesAsyncPager:
         r"""Lists composite indexes.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_list_indexes():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.ListIndexesRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_indexes(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.ListIndexesRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.ListIndexesRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.ListIndexes][google.firestore.admin.v1.FirestoreAdmin.ListIndexes].
             parent (:class:`str`):
@@ -312,7 +429,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -367,17 +484,36 @@ class FirestoreAdminAsyncClient:
 
     async def get_index(
         self,
-        request: firestore_admin.GetIndexRequest = None,
+        request: Union[firestore_admin.GetIndexRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> index.Index:
         r"""Gets a composite index.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_get_index():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.GetIndexRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_index(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.GetIndexRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.GetIndexRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.GetIndex][google.firestore.admin.v1.FirestoreAdmin.GetIndex].
             name (:class:`str`):
@@ -401,7 +537,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -450,17 +586,33 @@ class FirestoreAdminAsyncClient:
 
     async def delete_index(
         self,
-        request: firestore_admin.DeleteIndexRequest = None,
+        request: Union[firestore_admin.DeleteIndexRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Deletes a composite index.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_delete_index():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.DeleteIndexRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_index(request=request)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.DeleteIndexRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.DeleteIndexRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.DeleteIndex][google.firestore.admin.v1.FirestoreAdmin.DeleteIndex].
             name (:class:`str`):
@@ -477,7 +629,7 @@ class FirestoreAdminAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -525,17 +677,36 @@ class FirestoreAdminAsyncClient:
 
     async def get_field(
         self,
-        request: firestore_admin.GetFieldRequest = None,
+        request: Union[firestore_admin.GetFieldRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> field.Field:
         r"""Gets the metadata and configuration for a Field.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_get_field():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.GetFieldRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_field(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.GetFieldRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.GetFieldRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.GetField][google.firestore.admin.v1.FirestoreAdmin.GetField].
             name (:class:`str`):
@@ -561,7 +732,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -610,10 +781,10 @@ class FirestoreAdminAsyncClient:
 
     async def update_field(
         self,
-        request: firestore_admin.UpdateFieldRequest = None,
+        request: Union[firestore_admin.UpdateFieldRequest, dict] = None,
         *,
         field: gfa_field.Field = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -634,8 +805,35 @@ class FirestoreAdminAsyncClient:
         the special ``Field`` with resource name:
         ``projects/{project_id}/databases/{database_id}/collectionGroups/__default__/fields/*``.
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_update_field():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                field = firestore_admin_v1.Field()
+                field.name = "name_value"
+
+                request = firestore_admin_v1.UpdateFieldRequest(
+                    field=field,
+                )
+
+                # Make the request
+                operation = client.update_field(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.UpdateFieldRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.UpdateFieldRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.UpdateField][google.firestore.admin.v1.FirestoreAdmin.UpdateField].
             field (:class:`google.cloud.firestore_admin_v1.types.Field`):
@@ -663,7 +861,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([field])
         if request is not None and has_flattened_params:
@@ -711,10 +909,10 @@ class FirestoreAdminAsyncClient:
 
     async def list_fields(
         self,
-        request: firestore_admin.ListFieldsRequest = None,
+        request: Union[firestore_admin.ListFieldsRequest, dict] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListFieldsAsyncPager:
@@ -725,10 +923,32 @@ class FirestoreAdminAsyncClient:
         only supports listing fields that have been explicitly
         overridden. To issue this query, call
         [FirestoreAdmin.ListFields][google.firestore.admin.v1.FirestoreAdmin.ListFields]
-        with the filter set to ``indexConfig.usesAncestorConfig:false``.
+        with the filter set to ``indexConfig.usesAncestorConfig:false``
+        .
+
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_list_fields():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.ListFieldsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_fields(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
 
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.ListFieldsRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.ListFieldsRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.ListFields][google.firestore.admin.v1.FirestoreAdmin.ListFields].
             parent (:class:`str`):
@@ -754,7 +974,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -809,10 +1029,10 @@ class FirestoreAdminAsyncClient:
 
     async def export_documents(
         self,
-        request: firestore_admin.ExportDocumentsRequest = None,
+        request: Union[firestore_admin.ExportDocumentsRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -827,8 +1047,36 @@ class FirestoreAdminAsyncClient:
         before completion it may leave partial data behind in
         Google Cloud Storage.
 
+        For more details on export behavior and output format,
+        refer to:
+        https://cloud.google.com/firestore/docs/manage-data/export-import
+
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_export_documents():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.ExportDocumentsRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                operation = client.export_documents(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.ExportDocumentsRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.ExportDocumentsRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.ExportDocuments][google.firestore.admin.v1.FirestoreAdmin.ExportDocuments].
             name (:class:`str`):
@@ -856,7 +1104,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -902,10 +1150,10 @@ class FirestoreAdminAsyncClient:
 
     async def import_documents(
         self,
-        request: firestore_admin.ImportDocumentsRequest = None,
+        request: Union[firestore_admin.ImportDocumentsRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -917,8 +1165,32 @@ class FirestoreAdminAsyncClient:
         cancelled, it is possible that a subset of the data has
         already been imported to Cloud Firestore.
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_import_documents():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.ImportDocumentsRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                operation = client.import_documents(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_admin_v1.types.ImportDocumentsRequest`):
+            request (Union[google.cloud.firestore_admin_v1.types.ImportDocumentsRequest, dict]):
                 The request object. The request for
                 [FirestoreAdmin.ImportDocuments][google.firestore.admin.v1.FirestoreAdmin.ImportDocuments].
             name (:class:`str`):
@@ -954,7 +1226,7 @@ class FirestoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -998,6 +1270,305 @@ class FirestoreAdminAsyncClient:
         # Done; return the response.
         return response
 
+    async def get_database(
+        self,
+        request: Union[firestore_admin.GetDatabaseRequest, dict] = None,
+        *,
+        name: str = None,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
+        timeout: float = None,
+        metadata: Sequence[Tuple[str, str]] = (),
+    ) -> database.Database:
+        r"""Gets information about a database.
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_get_database():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.GetDatabaseRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_database(request=request)
+
+                # Handle the response
+                print(response)
+
+        Args:
+            request (Union[google.cloud.firestore_admin_v1.types.GetDatabaseRequest, dict]):
+                The request object. The request for
+                [FirestoreAdmin.GetDatabase][google.firestore.admin.v1.FirestoreAdmin.GetDatabase].
+            name (:class:`str`):
+                Required. A name of the form
+                ``projects/{project_id}/databases/{database_id}``
+
+                This corresponds to the ``name`` field
+                on the ``request`` instance; if ``request`` is provided, this
+                should not be set.
+            retry (google.api_core.retry.Retry): Designation of what errors, if any,
+                should be retried.
+            timeout (float): The timeout for this request.
+            metadata (Sequence[Tuple[str, str]]): Strings which should be
+                sent along with the request as metadata.
+
+        Returns:
+            google.cloud.firestore_admin_v1.types.Database:
+                A Cloud Firestore Database.
+                   Currently only one database is allowed per cloud
+                   project; this database must have a database_id of
+                   '(default)'.
+
+        """
+        # Create or coerce a protobuf request object.
+        # Quick check: If we got a request object, we should *not* have
+        # gotten any keyword arguments that map to the request.
+        has_flattened_params = any([name])
+        if request is not None and has_flattened_params:
+            raise ValueError(
+                "If the `request` argument is set, then none of "
+                "the individual field arguments should be set."
+            )
+
+        request = firestore_admin.GetDatabaseRequest(request)
+
+        # If we have keyword arguments corresponding to fields on the
+        # request, apply these.
+        if name is not None:
+            request.name = name
+
+        # Wrap the RPC method; this adds retry and timeout information,
+        # and friendly error handling.
+        rpc = gapic_v1.method_async.wrap_method(
+            self._client._transport.get_database,
+            default_timeout=None,
+            client_info=DEFAULT_CLIENT_INFO,
+        )
+
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
+        )
+
+        # Send the request.
+        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+
+        # Done; return the response.
+        return response
+
+    async def list_databases(
+        self,
+        request: Union[firestore_admin.ListDatabasesRequest, dict] = None,
+        *,
+        parent: str = None,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
+        timeout: float = None,
+        metadata: Sequence[Tuple[str, str]] = (),
+    ) -> firestore_admin.ListDatabasesResponse:
+        r"""List all the databases in the project.
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_list_databases():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.ListDatabasesRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.list_databases(request=request)
+
+                # Handle the response
+                print(response)
+
+        Args:
+            request (Union[google.cloud.firestore_admin_v1.types.ListDatabasesRequest, dict]):
+                The request object. A request to list the Firestore
+                Databases in all locations for a project.
+            parent (:class:`str`):
+                Required. A parent name of the form
+                ``projects/{project_id}``
+
+                This corresponds to the ``parent`` field
+                on the ``request`` instance; if ``request`` is provided, this
+                should not be set.
+            retry (google.api_core.retry.Retry): Designation of what errors, if any,
+                should be retried.
+            timeout (float): The timeout for this request.
+            metadata (Sequence[Tuple[str, str]]): Strings which should be
+                sent along with the request as metadata.
+
+        Returns:
+            google.cloud.firestore_admin_v1.types.ListDatabasesResponse:
+                The list of databases for a project.
+        """
+        # Create or coerce a protobuf request object.
+        # Quick check: If we got a request object, we should *not* have
+        # gotten any keyword arguments that map to the request.
+        has_flattened_params = any([parent])
+        if request is not None and has_flattened_params:
+            raise ValueError(
+                "If the `request` argument is set, then none of "
+                "the individual field arguments should be set."
+            )
+
+        request = firestore_admin.ListDatabasesRequest(request)
+
+        # If we have keyword arguments corresponding to fields on the
+        # request, apply these.
+        if parent is not None:
+            request.parent = parent
+
+        # Wrap the RPC method; this adds retry and timeout information,
+        # and friendly error handling.
+        rpc = gapic_v1.method_async.wrap_method(
+            self._client._transport.list_databases,
+            default_timeout=None,
+            client_info=DEFAULT_CLIENT_INFO,
+        )
+
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
+        )
+
+        # Send the request.
+        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+
+        # Done; return the response.
+        return response
+
+    async def update_database(
+        self,
+        request: Union[firestore_admin.UpdateDatabaseRequest, dict] = None,
+        *,
+        database: gfa_database.Database = None,
+        update_mask: field_mask_pb2.FieldMask = None,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
+        timeout: float = None,
+        metadata: Sequence[Tuple[str, str]] = (),
+    ) -> operation_async.AsyncOperation:
+        r"""Updates a database.
+
+        .. code-block:: python
+
+            from google.cloud import firestore_admin_v1
+
+            def sample_update_database():
+                # Create a client
+                client = firestore_admin_v1.FirestoreAdminClient()
+
+                # Initialize request argument(s)
+                request = firestore_admin_v1.UpdateDatabaseRequest(
+                )
+
+                # Make the request
+                operation = client.update_database(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
+        Args:
+            request (Union[google.cloud.firestore_admin_v1.types.UpdateDatabaseRequest, dict]):
+                The request object. The request for
+                [FirestoreAdmin.UpdateDatabase][google.firestore.admin.v1.FirestoreAdmin.UpdateDatabase].
+            database (:class:`google.cloud.firestore_admin_v1.types.Database`):
+                Required. The database to update.
+                This corresponds to the ``database`` field
+                on the ``request`` instance; if ``request`` is provided, this
+                should not be set.
+            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
+                The list of fields to be updated.
+                This corresponds to the ``update_mask`` field
+                on the ``request`` instance; if ``request`` is provided, this
+                should not be set.
+            retry (google.api_core.retry.Retry): Designation of what errors, if any,
+                should be retried.
+            timeout (float): The timeout for this request.
+            metadata (Sequence[Tuple[str, str]]): Strings which should be
+                sent along with the request as metadata.
+
+        Returns:
+            google.api_core.operation_async.AsyncOperation:
+                An object representing a long-running operation.
+
+                The result type for the operation will be :class:`google.cloud.firestore_admin_v1.types.Database` A Cloud Firestore Database.
+                   Currently only one database is allowed per cloud
+                   project; this database must have a database_id of
+                   '(default)'.
+
+        """
+        # Create or coerce a protobuf request object.
+        # Quick check: If we got a request object, we should *not* have
+        # gotten any keyword arguments that map to the request.
+        has_flattened_params = any([database, update_mask])
+        if request is not None and has_flattened_params:
+            raise ValueError(
+                "If the `request` argument is set, then none of "
+                "the individual field arguments should be set."
+            )
+
+        request = firestore_admin.UpdateDatabaseRequest(request)
+
+        # If we have keyword arguments corresponding to fields on the
+        # request, apply these.
+        if database is not None:
+            request.database = database
+        if update_mask is not None:
+            request.update_mask = update_mask
+
+        # Wrap the RPC method; this adds retry and timeout information,
+        # and friendly error handling.
+        rpc = gapic_v1.method_async.wrap_method(
+            self._client._transport.update_database,
+            default_timeout=None,
+            client_info=DEFAULT_CLIENT_INFO,
+        )
+
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("database.name", request.database.name),)
+            ),
+        )
+
+        # Send the request.
+        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+
+        # Wrap the response in an operation future.
+        response = operation_async.from_gapic(
+            response,
+            self._client._transport.operations_client,
+            gfa_database.Database,
+            metadata_type=firestore_admin.UpdateDatabaseMetadata,
+        )
+
+        # Done; return the response.
+        return response
+
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
