@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,16 @@
 import warnings
 from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
 
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import grpc_helpers_async  # type: ignore
-from google.api_core import operations_v1  # type: ignore
+from google.api_core import gapic_v1
+from google.api_core import grpc_helpers_async
+from google.api_core import operations_v1
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
-import packaging.version
 
 import grpc  # type: ignore
 from grpc.experimental import aio  # type: ignore
 
+from google.cloud.firestore_admin_v1.types import database
 from google.cloud.firestore_admin_v1.types import field
 from google.cloud.firestore_admin_v1.types import firestore_admin
 from google.cloud.firestore_admin_v1.types import index
@@ -38,6 +38,35 @@ from .grpc import FirestoreAdminGrpcTransport
 class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
     """gRPC AsyncIO backend transport for FirestoreAdmin.
 
+    The Cloud Firestore Admin API.
+
+    This API provides several administrative services for Cloud
+    Firestore.
+
+    Project, Database, Namespace, Collection, Collection Group, and
+    Document are used as defined in the Google Cloud Firestore API.
+
+    Operation: An Operation represents work being performed in the
+    background.
+
+    The index service manages Cloud Firestore indexes.
+
+    Index creation is performed asynchronously. An Operation resource is
+    created for each such asynchronous operation. The state of the
+    operation (including any errors encountered) may be queried via the
+    Operation resource.
+
+    The Operations collection provides a record of actions performed for
+    the specified Project (including any Operations in progress).
+    Operations are not created directly but through calls on other
+    collections or resources.
+
+    An Operation that is done may be deleted so that it is no longer
+    listed as part of the Operation collection. Operations are garbage
+    collected after 30 days. By default, ListOperations will only return
+    in progress and failed operations. To list completed operation,
+    issue a ListOperations request with the filter ``done: true``.
+
     Operations are created by service ``FirestoreAdmin``, but are
     accessed via service ``google.longrunning.Operations``.
 
@@ -133,16 +162,16 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                 If provided, it overrides the ``host`` argument and tries to create
                 a mutual TLS channel with client SSL credentials from
-                ``client_cert_source`` or applicatin default SSL credentials.
+                ``client_cert_source`` or application default SSL credentials.
             client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                 Deprecated. A callback to provide client SSL certificate bytes and
                 private key bytes, both in PEM format. It is ignored if
                 ``api_mtls_endpoint`` is None.
             ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
-                for grpc channel. It is ignored if ``channel`` is provided.
+                for the grpc channel. It is ignored if ``channel`` is provided.
             client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                 A callback to provide client certificate bytes and private key bytes,
-                both in PEM format. It is used to configure mutual TLS channel. It is
+                both in PEM format. It is used to configure a mutual TLS channel. It is
                 ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
@@ -163,7 +192,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         self._grpc_channel = None
         self._ssl_channel_credentials = ssl_channel_credentials
         self._stubs: Dict[str, Callable] = {}
-        self._operations_client = None
+        self._operations_client: Optional[operations_v1.OperationsAsyncClient] = None
 
         if api_mtls_endpoint:
             warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
@@ -211,8 +240,11 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         if not self._grpc_channel:
             self._grpc_channel = type(self).create_channel(
                 self._host,
+                # use the credentials which are saved
                 credentials=self._credentials,
-                credentials_file=credentials_file,
+                # Set ``credentials_file`` to ``None`` here as
+                # the credentials that we saved earlier should be used.
+                credentials_file=None,
                 scopes=self._scopes,
                 ssl_credentials=self._ssl_channel_credentials,
                 quota_project_id=quota_project_id,
@@ -242,7 +274,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         This property caches on the instance; repeated calls return the same
         client.
         """
-        # Sanity check: Only create a new client if we do not already have one.
+        # Quick check: Only create a new client if we do not already have one.
         if self._operations_client is None:
             self._operations_client = operations_v1.OperationsAsyncClient(
                 self.grpc_channel
@@ -449,7 +481,8 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         only supports listing fields that have been explicitly
         overridden. To issue this query, call
         [FirestoreAdmin.ListFields][google.firestore.admin.v1.FirestoreAdmin.ListFields]
-        with the filter set to ``indexConfig.usesAncestorConfig:false``.
+        with the filter set to ``indexConfig.usesAncestorConfig:false``
+        .
 
         Returns:
             Callable[[~.ListFieldsRequest],
@@ -488,6 +521,10 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         before completion it may leave partial data behind in
         Google Cloud Storage.
 
+        For more details on export behavior and output format,
+        refer to:
+        https://cloud.google.com/firestore/docs/manage-data/export-import
+
         Returns:
             Callable[[~.ExportDocumentsRequest],
                     Awaitable[~.Operation]]:
@@ -540,5 +577,91 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             )
         return self._stubs["import_documents"]
 
+    @property
+    def get_database(
+        self,
+    ) -> Callable[[firestore_admin.GetDatabaseRequest], Awaitable[database.Database]]:
+        r"""Return a callable for the get database method over gRPC.
+
+        Gets information about a database.
+
+        Returns:
+            Callable[[~.GetDatabaseRequest],
+                    Awaitable[~.Database]]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "get_database" not in self._stubs:
+            self._stubs["get_database"] = self.grpc_channel.unary_unary(
+                "/google.firestore.admin.v1.FirestoreAdmin/GetDatabase",
+                request_serializer=firestore_admin.GetDatabaseRequest.serialize,
+                response_deserializer=database.Database.deserialize,
+            )
+        return self._stubs["get_database"]
+
+    @property
+    def list_databases(
+        self,
+    ) -> Callable[
+        [firestore_admin.ListDatabasesRequest],
+        Awaitable[firestore_admin.ListDatabasesResponse],
+    ]:
+        r"""Return a callable for the list databases method over gRPC.
+
+        List all the databases in the project.
+
+        Returns:
+            Callable[[~.ListDatabasesRequest],
+                    Awaitable[~.ListDatabasesResponse]]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "list_databases" not in self._stubs:
+            self._stubs["list_databases"] = self.grpc_channel.unary_unary(
+                "/google.firestore.admin.v1.FirestoreAdmin/ListDatabases",
+                request_serializer=firestore_admin.ListDatabasesRequest.serialize,
+                response_deserializer=firestore_admin.ListDatabasesResponse.deserialize,
+            )
+        return self._stubs["list_databases"]
+
+    @property
+    def update_database(
+        self,
+    ) -> Callable[
+        [firestore_admin.UpdateDatabaseRequest], Awaitable[operations_pb2.Operation]
+    ]:
+        r"""Return a callable for the update database method over gRPC.
+
+        Updates a database.
+
+        Returns:
+            Callable[[~.UpdateDatabaseRequest],
+                    Awaitable[~.Operation]]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "update_database" not in self._stubs:
+            self._stubs["update_database"] = self.grpc_channel.unary_unary(
+                "/google.firestore.admin.v1.FirestoreAdmin/UpdateDatabase",
+                request_serializer=firestore_admin.UpdateDatabaseRequest.serialize,
+                response_deserializer=operations_pb2.Operation.FromString,
+            )
+        return self._stubs["update_database"]
+
+    def close(self):
+        return self.grpc_channel.close()
+
 
 __all__ = ("FirestoreAdminGrpcAsyncIOTransport",)
