@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,18 +13,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+from .database import Database
 from .field import Field
 from .firestore_admin import (
     CreateIndexRequest,
     DeleteIndexRequest,
     ExportDocumentsRequest,
+    GetDatabaseRequest,
     GetFieldRequest,
     GetIndexRequest,
     ImportDocumentsRequest,
+    ListDatabasesRequest,
+    ListDatabasesResponse,
     ListFieldsRequest,
     ListFieldsResponse,
     ListIndexesRequest,
     ListIndexesResponse,
+    UpdateDatabaseMetadata,
+    UpdateDatabaseRequest,
     UpdateFieldRequest,
 )
 from .index import Index
@@ -40,17 +46,23 @@ from .operation import (
 )
 
 __all__ = (
+    "Database",
     "Field",
     "CreateIndexRequest",
     "DeleteIndexRequest",
     "ExportDocumentsRequest",
+    "GetDatabaseRequest",
     "GetFieldRequest",
     "GetIndexRequest",
     "ImportDocumentsRequest",
+    "ListDatabasesRequest",
+    "ListDatabasesResponse",
     "ListFieldsRequest",
     "ListFieldsResponse",
     "ListIndexesRequest",
     "ListIndexesResponse",
+    "UpdateDatabaseMetadata",
+    "UpdateDatabaseRequest",
     "UpdateFieldRequest",
     "Index",
     "LocationMetadata",
