@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,11 +34,16 @@ __protobuf__ = proto.module(
 
 class BundledQuery(proto.Message):
     r"""Encodes a query saved in the bundle.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         parent (str):
             The parent resource name.
         structured_query (google.firestore.v1.query_pb2.StructuredQuery):
             A structured query.
+
+            This field is a member of `oneof`_ ``query_type``.
         limit_type (google.cloud.bundle.types.BundledQuery.LimitType):
 
     """
@@ -84,6 +89,7 @@ class NamedQuery(proto.Message):
 
 class BundledDocumentMetadata(proto.Message):
     r"""Metadata describing a Firestore document saved in the bundle.
+
     Attributes:
         name (str):
             The document key of a bundled document.
@@ -105,6 +111,7 @@ class BundledDocumentMetadata(proto.Message):
 
 class BundleMetadata(proto.Message):
     r"""Metadata describing the bundle file/stream.
+
     Attributes:
         id (str):
             The ID of the bundle.
@@ -134,15 +141,26 @@ class BundleElement(proto.Message):
     follow after ``metadata``. Every ``document_metadata`` is
     immediately followed by a ``document``.
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         metadata (google.cloud.bundle.types.BundleMetadata):
 
+            This field is a member of `oneof`_ ``element_type``.
         named_query (google.cloud.bundle.types.NamedQuery):
 
+            This field is a member of `oneof`_ ``element_type``.
         document_metadata (google.cloud.bundle.types.BundledDocumentMetadata):
 
+            This field is a member of `oneof`_ ``element_type``.
         document (google.firestore.v1.document_pb2.Document):
 
+            This field is a member of `oneof`_ ``element_type``.
     """
 
     metadata = proto.Field(
