@@ -14,8 +14,8 @@
 
 """Classes for representing collections for the Google Cloud Firestore API."""
 
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 
 from google.cloud.firestore_v1.base_collection import (
     BaseCollectionReference,
@@ -107,8 +107,8 @@ class AsyncCollectionReference(BaseCollectionReference):
                 * A document reference for the created document.
 
         Raises:
-            ~google.cloud.exceptions.Conflict: If ``document_id`` is provided
-                and the document already exists.
+            :class:`google.cloud.exceptions.Conflict`:
+                If ``document_id`` is provided and the document already exists.
         """
         document_ref, kwargs = self._prep_add(
             document_data, document_id, retry, timeout,
