@@ -252,7 +252,7 @@ class BulkWriter(AsyncBulkWriterMixin):
 
     def __init__(
         self,
-        client: Optional["BaseClient"] = None,
+        client: "BaseClient" = None,
         options: Optional["BulkWriterOptions"] = None,
     ):
         # Because `BulkWriter` instances are all synchronous/blocking on the
@@ -895,6 +895,11 @@ except ImportError:
             self.mode = mode
             self.retry = retry
 
+        def __eq__(self, other):
+            if not isinstance(other, self.__class__):  # pragma: NO COVER
+                return NotImplemented
+            return self.__dict__ == other.__dict__
+
     class BulkWriteFailure:
         def __init__(
             self,
