@@ -24,8 +24,8 @@ In the hierarchy of API concepts
   :class:`~google.cloud.firestore_v1.document.DocumentReference`
 """
 
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 
 from google.cloud.firestore_v1.base_client import (
     BaseClient,
@@ -50,7 +50,6 @@ from typing import Any, Generator, Iterable, List, Optional, Union, TYPE_CHECKIN
 # Types needed only for Type Hints
 from google.cloud.firestore_v1.base_document import DocumentSnapshot
 
-
 if TYPE_CHECKING:
     from google.cloud.firestore_v1.bulk_writer import BulkWriter  # pragma: NO COVER
 
@@ -319,22 +318,20 @@ class Client(BaseClient):
                 if you want to override the default throttling behavior.
 
         """
-        return self._recursive_delete(
-            reference, bulk_writer=bulk_writer, chunk_size=chunk_size,
-        )
+        if bulk_writer is None:
+            bulk_writer or self.bulk_writer()
+
+        return self._recursive_delete(reference, bulk_writer, chunk_size=chunk_size,)
 
     def _recursive_delete(
         self,
         reference: Union[CollectionReference, DocumentReference],
+        bulk_writer: "BulkWriter",
         *,
-        bulk_writer: Optional["BulkWriter"] = None,
         chunk_size: Optional[int] = 5000,
         depth: Optional[int] = 0,
     ) -> int:
         """Recursion helper for `recursive_delete."""
-        from google.cloud.firestore_v1.bulk_writer import BulkWriter
-
-        bulk_writer = bulk_writer or BulkWriter()
 
         num_deleted: int = 0
 
@@ -354,10 +351,7 @@ class Client(BaseClient):
             col_ref: CollectionReference
             for col_ref in reference.collections():
                 num_deleted += self._recursive_delete(
-                    col_ref,
-                    bulk_writer=bulk_writer,
-                    chunk_size=chunk_size,
-                    depth=depth + 1,
+                    col_ref, bulk_writer, chunk_size=chunk_size, depth=depth + 1,
                 )
             num_deleted += 1
             bulk_writer.delete(reference)
