@@ -14,8 +14,8 @@
 
 """Classes for representing collections for the Google Cloud Firestore API."""
 
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 
 from google.cloud.firestore_v1.base_collection import (
     BaseCollectionReference,
@@ -98,8 +98,8 @@ class CollectionReference(BaseCollectionReference):
                 * A document reference for the created document.
 
         Raises:
-            ~google.cloud.exceptions.Conflict: If ``document_id`` is provided
-                and the document already exists.
+            :class:`google.cloud.exceptions.Conflict`:
+                If ``document_id`` is provided and the document already exists.
         """
         document_ref, kwargs = self._prep_add(
             document_data, document_id, retry, timeout,
@@ -237,9 +237,5 @@ class CollectionReference(BaseCollectionReference):
             # Terminate this watch
             collection_watch.unsubscribe()
         """
-        return Watch.for_query(
-            self._query(),
-            callback,
-            document.DocumentSnapshot,
-            document.DocumentReference,
-        )
+        query = self._query()
+        return Watch.for_query(query, callback, document.DocumentSnapshot)
