@@ -16,8 +16,8 @@
 import datetime
 import logging
 
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.cloud._helpers import _datetime_to_pb_timestamp  # type: ignore
 
 from google.cloud.firestore_v1.base_document import (
@@ -92,7 +92,7 @@ class DocumentReference(BaseDocumentReference):
                 A write result contains an ``update_time`` field.
 
         Raises:
-            :class:`~google.cloud.exceptions.Conflict`:
+            :class:`google.cloud.exceptions.Conflict`:
                 If the document already exists.
         """
         batch, kwargs = self._prep_create(document_data, retry, timeout)
@@ -318,7 +318,8 @@ class DocumentReference(BaseDocumentReference):
             result contains an ``update_time`` field.
 
         Raises:
-            ~google.cloud.exceptions.NotFound: If the document does not exist.
+            :class:`google.cloud.exceptions.NotFound`:
+                If the document does not exist.
         """
         batch, kwargs = self._prep_update(field_updates, option, retry, timeout)
         write_results = batch.commit(**kwargs)
@@ -488,4 +489,4 @@ class DocumentReference(BaseDocumentReference):
             # Terminate this watch
             doc_watch.unsubscribe()
         """
-        return Watch.for_document(self, callback, DocumentSnapshot, DocumentReference)
+        return Watch.for_document(self, callback, DocumentSnapshot)
