@@ -20,8 +20,9 @@ a more common way to create a query than direct usage of the constructor.
 """
 from google.cloud import firestore_v1
 from google.cloud.firestore_v1.base_document import DocumentSnapshot
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 
 from google.cloud.firestore_v1.base_query import (
     BaseCollectionGroup,
@@ -170,12 +171,6 @@ class Query(BaseQuery):
     def _chunkify(
         self, chunk_size: int
     ) -> Generator[List[DocumentSnapshot], None, None]:
-        # Catch the edge case where a developer writes the following:
-        # `my_query.limit(500)._chunkify(1000)`, which ultimately nullifies any
-        # need to yield chunks.
-        if self._limit and chunk_size > self._limit:
-            yield self.get()
-            return
 
         max_to_return: Optional[int] = self._limit
         num_returned: int = 0
@@ -190,11 +185,15 @@ class Query(BaseQuery):
             # Apply the optionally pruned limit and the cursor, if we are past
             # the first page.
             _q = original.limit(_chunk_size)
+
             if last_document:
                 _q = _q.start_after(last_document)
 
             snapshots = _q.get()
-            last_document = snapshots[-1]
+
+            if snapshots:
+                last_document = snapshots[-1]
+
             num_returned += len(snapshots)
 
             yield snapshots
@@ -208,6 +207,29 @@ class Query(BaseQuery):
             ):
                 return
 
+    def _get_stream_iterator(self, transaction, retry, timeout):
+        """Helper method for :meth:`stream`."""
+        request, expected_prefix, kwargs = self._prep_stream(
+            transaction, retry, timeout,
+        )
+
+        response_iterator = self._client._firestore_api.run_query(
+            request=request, metadata=self._client._rpc_metadata, **kwargs,
+        )
+
+        return response_iterator, expected_prefix
+
+    def _retry_query_after_exception(self, exc, retry, transaction):
+        """Helper method for :meth:`stream`."""
+        if transaction is None:  # no snapshot-based retry inside transaction
+            if retry is gapic_v1.method.DEFAULT:
+                transport = self._client._firestore_api._transport
+                gapic_callable = transport.run_query
+                retry = gapic_callable._retry
+            return retry._predicate(exc)
+
+        return False
+
     def stream(
         self,
         transaction=None,
@@ -244,15 +266,28 @@ class Query(BaseQuery):
             :class:`~google.cloud.firestore_v1.document.DocumentSnapshot`:
             The next document that fulfills the query.
         """
-        request, expected_prefix, kwargs = self._prep_stream(
+        response_iterator, expected_prefix = self._get_stream_iterator(
             transaction, retry, timeout,
         )
 
-        response_iterator = self._client._firestore_api.run_query(
-            request=request, metadata=self._client._rpc_metadata, **kwargs,
-        )
+        last_snapshot = None
+
+        while True:
+            try:
+                response = next(response_iterator, None)
+            except exceptions.GoogleAPICallError as exc:
+                if self._retry_query_after_exception(exc, retry, transaction):
+                    new_query = self.start_after(last_snapshot)
+                    response_iterator, _ = new_query._get_stream_iterator(
+                        transaction, retry, timeout,
+                    )
+                    continue
+                else:
+                    raise
+
+            if response is None:  # EOI
+                break
 
-        for response in response_iterator:
             if self._all_descendants:
                 snapshot = _collection_group_query_response_to_snapshot(
                     response, self._parent
@@ -262,6 +297,7 @@ class Query(BaseQuery):
                     response, self._parent, expected_prefix
                 )
             if snapshot is not None:
+                last_snapshot = snapshot
                 yield snapshot
 
     def on_snapshot(self, callback: Callable) -> Watch:
@@ -293,9 +329,7 @@ class Query(BaseQuery):
             # Terminate this watch
             query_watch.unsubscribe()
         """
-        return Watch.for_query(
-            self, callback, document.DocumentSnapshot, document.DocumentReference
-        )
+        return Watch.for_query(self, callback, document.DocumentSnapshot)
 
     @staticmethod
     def _get_collection_reference_class() -> Type[
