@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ import functools
 import re
 from typing import (
     Dict,
+    Optional,
     AsyncIterable,
     Awaitable,
     AsyncIterator,
@@ -28,13 +29,18 @@ from typing import (
 )
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.firestore_v1.services.firestore import pagers
 from google.cloud.firestore_v1.types import common
 from google.cloud.firestore_v1.types import document
@@ -51,13 +57,13 @@ from .client import FirestoreClient
 
 class FirestoreAsyncClient:
     """The Cloud Firestore service.
-    Cloud Firestore is a fast, fully managed, serverless, cloud-
-    native NoSQL document database that simplifies storing, syncing,
-    and querying data for your mobile, web, and IoT apps at global
-    scale. Its client libraries provide live synchronization and
-    offline support, while its security features and integrations
-    with Firebase and Google Cloud Platform (GCP) accelerate
-    building truly serverless apps.
+    Cloud Firestore is a fast, fully managed, serverless,
+    cloud-native NoSQL document database that simplifies storing,
+    syncing, and querying data for your mobile, web, and IoT apps at
+    global scale. Its client libraries provide live synchronization
+    and offline support, while its security features and
+    integrations with Firebase and Google Cloud Platform (GCP)
+    accelerate building truly serverless apps.
     """
 
     _client: FirestoreClient
@@ -117,6 +123,42 @@ class FirestoreAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return FirestoreClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> FirestoreTransport:
         """Returns the transport used by the client instance.
@@ -179,16 +221,36 @@ class FirestoreAsyncClient:
 
     async def get_document(
         self,
-        request: firestore.GetDocumentRequest = None,
+        request: Union[firestore.GetDocumentRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> document.Document:
         r"""Gets a single document.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_get_document():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.GetDocumentRequest(
+                    transaction=b'transaction_blob',
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_document(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.GetDocumentRequest`):
+            request (Union[google.cloud.firestore_v1.types.GetDocumentRequest, dict]):
                 The request object. The request for
                 [Firestore.GetDocument][google.firestore.v1.Firestore.GetDocument].
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -240,16 +302,38 @@ class FirestoreAsyncClient:
 
     async def list_documents(
         self,
-        request: firestore.ListDocumentsRequest = None,
+        request: Union[firestore.ListDocumentsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListDocumentsAsyncPager:
         r"""Lists documents.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_list_documents():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.ListDocumentsRequest(
+                    transaction=b'transaction_blob',
+                    parent="parent_value",
+                    collection_id="collection_id_value",
+                )
+
+                # Make the request
+                page_result = client.list_documents(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.ListDocumentsRequest`):
+            request (Union[google.cloud.firestore_v1.types.ListDocumentsRequest, dict]):
                 The request object. The request for
                 [Firestore.ListDocuments][google.firestore.v1.Firestore.ListDocuments].
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -310,18 +394,36 @@ class FirestoreAsyncClient:
 
     async def update_document(
         self,
-        request: firestore.UpdateDocumentRequest = None,
+        request: Union[firestore.UpdateDocumentRequest, dict] = None,
         *,
         document: gf_document.Document = None,
         update_mask: common.DocumentMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> gf_document.Document:
         r"""Updates or inserts a document.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_update_document():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.UpdateDocumentRequest(
+                )
+
+                # Make the request
+                response = client.update_document(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.UpdateDocumentRequest`):
+            request (Union[google.cloud.firestore_v1.types.UpdateDocumentRequest, dict]):
                 The request object. The request for
                 [Firestore.UpdateDocument][google.firestore.v1.Firestore.UpdateDocument].
             document (:class:`google.cloud.firestore_v1.types.Document`):
@@ -359,7 +461,7 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([document, update_mask])
         if request is not None and has_flattened_params:
@@ -411,17 +513,33 @@ class FirestoreAsyncClient:
 
     async def delete_document(
         self,
-        request: firestore.DeleteDocumentRequest = None,
+        request: Union[firestore.DeleteDocumentRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Deletes a document.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_delete_document():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.DeleteDocumentRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_document(request=request)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.DeleteDocumentRequest`):
+            request (Union[google.cloud.firestore_v1.types.DeleteDocumentRequest, dict]):
                 The request object. The request for
                 [Firestore.DeleteDocument][google.firestore.v1.Firestore.DeleteDocument].
             name (:class:`str`):
@@ -439,7 +557,7 @@ class FirestoreAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -488,9 +606,9 @@ class FirestoreAsyncClient:
 
     def batch_get_documents(
         self,
-        request: firestore.BatchGetDocumentsRequest = None,
+        request: Union[firestore.BatchGetDocumentsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> Awaitable[AsyncIterable[firestore.BatchGetDocumentsResponse]]:
@@ -498,8 +616,30 @@ class FirestoreAsyncClient:
         Documents returned by this method are not guaranteed to
         be returned in the same order that they were requested.
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_batch_get_documents():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.BatchGetDocumentsRequest(
+                    transaction=b'transaction_blob',
+                    database="database_value",
+                )
+
+                # Make the request
+                stream = client.batch_get_documents(request=request)
+
+                # Handle the response
+                for response in stream:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.BatchGetDocumentsRequest`):
+            request (Union[google.cloud.firestore_v1.types.BatchGetDocumentsRequest, dict]):
                 The request object. The request for
                 [Firestore.BatchGetDocuments][google.firestore.v1.Firestore.BatchGetDocuments].
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -551,17 +691,36 @@ class FirestoreAsyncClient:
 
     async def begin_transaction(
         self,
-        request: firestore.BeginTransactionRequest = None,
+        request: Union[firestore.BeginTransactionRequest, dict] = None,
         *,
         database: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> firestore.BeginTransactionResponse:
         r"""Starts a new transaction.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_begin_transaction():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.BeginTransactionRequest(
+                    database="database_value",
+                )
+
+                # Make the request
+                response = client.begin_transaction(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.BeginTransactionRequest`):
+            request (Union[google.cloud.firestore_v1.types.BeginTransactionRequest, dict]):
                 The request object. The request for
                 [Firestore.BeginTransaction][google.firestore.v1.Firestore.BeginTransaction].
             database (:class:`str`):
@@ -584,7 +743,7 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([database])
         if request is not None and has_flattened_params:
@@ -634,19 +793,39 @@ class FirestoreAsyncClient:
 
     async def commit(
         self,
-        request: firestore.CommitRequest = None,
+        request: Union[firestore.CommitRequest, dict] = None,
         *,
         database: str = None,
         writes: Sequence[gf_write.Write] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> firestore.CommitResponse:
         r"""Commits a transaction, while optionally updating
         documents.
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_commit():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.CommitRequest(
+                    database="database_value",
+                )
+
+                # Make the request
+                response = client.commit(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.CommitRequest`):
+            request (Union[google.cloud.firestore_v1.types.CommitRequest, dict]):
                 The request object. The request for
                 [Firestore.Commit][google.firestore.v1.Firestore.Commit].
             database (:class:`str`):
@@ -676,7 +855,7 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([database, writes])
         if request is not None and has_flattened_params:
@@ -726,18 +905,35 @@ class FirestoreAsyncClient:
 
     async def rollback(
         self,
-        request: firestore.RollbackRequest = None,
+        request: Union[firestore.RollbackRequest, dict] = None,
         *,
         database: str = None,
         transaction: bytes = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Rolls back a transaction.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_rollback():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.RollbackRequest(
+                    database="database_value",
+                    transaction=b'transaction_blob',
+                )
+
+                # Make the request
+                client.rollback(request=request)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.RollbackRequest`):
+            request (Union[google.cloud.firestore_v1.types.RollbackRequest, dict]):
                 The request object. The request for
                 [Firestore.Rollback][google.firestore.v1.Firestore.Rollback].
             database (:class:`str`):
@@ -761,7 +957,7 @@ class FirestoreAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([database, transaction])
         if request is not None and has_flattened_params:
@@ -812,16 +1008,37 @@ class FirestoreAsyncClient:
 
     def run_query(
         self,
-        request: firestore.RunQueryRequest = None,
+        request: Union[firestore.RunQueryRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> Awaitable[AsyncIterable[firestore.RunQueryResponse]]:
         r"""Runs a query.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_run_query():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.RunQueryRequest(
+                    transaction=b'transaction_blob',
+                    parent="parent_value",
+                )
+
+                # Make the request
+                stream = client.run_query(request=request)
+
+                # Handle the response
+                for response in stream:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.RunQueryRequest`):
+            request (Union[google.cloud.firestore_v1.types.RunQueryRequest, dict]):
                 The request object. The request for
                 [Firestore.RunQuery][google.firestore.v1.Firestore.RunQuery].
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -873,9 +1090,9 @@ class FirestoreAsyncClient:
 
     async def partition_query(
         self,
-        request: firestore.PartitionQueryRequest = None,
+        request: Union[firestore.PartitionQueryRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.PartitionQueryAsyncPager:
@@ -885,8 +1102,29 @@ class FirestoreAsyncClient:
         used by RunQuery as starting/end points for the query
         results.
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_partition_query():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.PartitionQueryRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.partition_query(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.PartitionQueryRequest`):
+            request (Union[google.cloud.firestore_v1.types.PartitionQueryRequest, dict]):
                 The request object. The request for
                 [Firestore.PartitionQuery][google.firestore.v1.Firestore.PartitionQuery].
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -949,13 +1187,44 @@ class FirestoreAsyncClient:
         self,
         requests: AsyncIterator[firestore.WriteRequest] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> Awaitable[AsyncIterable[firestore.WriteResponse]]:
         r"""Streams batches of document updates and deletes, in
         order.
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_write():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.WriteRequest(
+                    database="database_value",
+                )
+
+                # This method expects an iterator which contains
+                # 'firestore_v1.WriteRequest' objects
+                # Here we create a generator that yields a single `request` for
+                # demonstrative purposes.
+                requests = [request]
+
+                def request_generator():
+                    for request in requests:
+                        yield request
+
+                # Make the request
+                stream = client.write(requests=request_generator())
+
+                # Handle the response
+                for response in stream:
+                    print(response)
+
         Args:
             requests (AsyncIterator[`google.cloud.firestore_v1.types.WriteRequest`]):
                 The request object AsyncIterator. The request for
@@ -1005,12 +1274,46 @@ class FirestoreAsyncClient:
         self,
         requests: AsyncIterator[firestore.ListenRequest] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> Awaitable[AsyncIterable[firestore.ListenResponse]]:
         r"""Listens to changes.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_listen():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                add_target = firestore_v1.Target()
+                add_target.resume_token = b'resume_token_blob'
+
+                request = firestore_v1.ListenRequest(
+                    add_target=add_target,
+                    database="database_value",
+                )
+
+                # This method expects an iterator which contains
+                # 'firestore_v1.ListenRequest' objects
+                # Here we create a generator that yields a single `request` for
+                # demonstrative purposes.
+                requests = [request]
+
+                def request_generator():
+                    for request in requests:
+                        yield request
+
+                # Make the request
+                stream = client.listen(requests=request_generator())
+
+                # Handle the response
+                for response in stream:
+                    print(response)
+
         Args:
             requests (AsyncIterator[`google.cloud.firestore_v1.types.ListenRequest`]):
                 The request object AsyncIterator. A request for
@@ -1060,17 +1363,37 @@ class FirestoreAsyncClient:
 
     async def list_collection_ids(
         self,
-        request: firestore.ListCollectionIdsRequest = None,
+        request: Union[firestore.ListCollectionIdsRequest, dict] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListCollectionIdsAsyncPager:
         r"""Lists all the collection IDs underneath a document.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_list_collection_ids():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.ListCollectionIdsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_collection_ids(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.ListCollectionIdsRequest`):
+            request (Union[google.cloud.firestore_v1.types.ListCollectionIdsRequest, dict]):
                 The request object. The request for
                 [Firestore.ListCollectionIds][google.firestore.v1.Firestore.ListCollectionIds].
             parent (:class:`str`):
@@ -1098,7 +1421,7 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -1154,9 +1477,9 @@ class FirestoreAsyncClient:
 
     async def batch_write(
         self,
-        request: firestore.BatchWriteRequest = None,
+        request: Union[firestore.BatchWriteRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> firestore.BatchWriteResponse:
@@ -1172,8 +1495,28 @@ class FirestoreAsyncClient:
         If you require an atomically applied set of writes, use
         [Commit][google.firestore.v1.Firestore.Commit] instead.
 
+
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_batch_write():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.BatchWriteRequest(
+                    database="database_value",
+                )
+
+                # Make the request
+                response = client.batch_write(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.BatchWriteRequest`):
+            request (Union[google.cloud.firestore_v1.types.BatchWriteRequest, dict]):
                 The request object. The request for
                 [Firestore.BatchWrite][google.firestore.v1.Firestore.BatchWrite].
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -1224,16 +1567,36 @@ class FirestoreAsyncClient:
 
     async def create_document(
         self,
-        request: firestore.CreateDocumentRequest = None,
+        request: Union[firestore.CreateDocumentRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> document.Document:
         r"""Creates a new document.
 
+        .. code-block:: python
+
+            from google.cloud import firestore_v1
+
+            def sample_create_document():
+                # Create a client
+                client = firestore_v1.FirestoreClient()
+
+                # Initialize request argument(s)
+                request = firestore_v1.CreateDocumentRequest(
+                    parent="parent_value",
+                    collection_id="collection_id_value",
+                )
+
+                # Make the request
+                response = client.create_document(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.firestore_v1.types.CreateDocumentRequest`):
+            request (Union[google.cloud.firestore_v1.types.CreateDocumentRequest, dict]):
                 The request object. The request for
                 [Firestore.CreateDocument][google.firestore.v1.Firestore.CreateDocument].
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -1281,6 +1644,12 @@ class FirestoreAsyncClient:
         # Done; return the response.
         return response
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
