@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -61,6 +61,13 @@ class GetDocumentRequest(proto.Message):
     r"""The request for
     [Firestore.GetDocument][google.firestore.v1.Firestore.GetDocument].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         name (str):
             Required. The resource name of the Document to get. In the
@@ -74,10 +81,14 @@ class GetDocumentRequest(proto.Message):
             the response.
         transaction (bytes):
             Reads the document in a transaction.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             Reads the version of the document at the
             given time. This may not be older than 270
             seconds.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
     """
 
     name = proto.Field(proto.STRING, number=1,)
@@ -95,6 +106,13 @@ class ListDocumentsRequest(proto.Message):
     r"""The request for
     [Firestore.ListDocuments][google.firestore.v1.Firestore.ListDocuments].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         parent (str):
             Required. The parent resource name. In the format:
@@ -123,9 +141,13 @@ class ListDocumentsRequest(proto.Message):
             the response.
         transaction (bytes):
             Reads documents in a transaction.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             Reads documents as they were at the given
             time. This may not be older than 270 seconds.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
         show_missing (bool):
             If the list should show missing documents. A missing
             document is a document that does not exist but has
@@ -276,6 +298,13 @@ class BatchGetDocumentsRequest(proto.Message):
     r"""The request for
     [Firestore.BatchGetDocuments][google.firestore.v1.Firestore.BatchGetDocuments].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         database (str):
             Required. The database name. In the format:
@@ -294,14 +323,20 @@ class BatchGetDocumentsRequest(proto.Message):
             the response.
         transaction (bytes):
             Reads documents in a transaction.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
         new_transaction (google.cloud.firestore_v1.types.TransactionOptions):
             Starts a new transaction and reads the
             documents. Defaults to a read-only transaction.
             The new transaction ID will be returned as the
             first response in the stream.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             Reads documents as they were at the given
             time. This may not be older than 270 seconds.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
     """
 
     database = proto.Field(proto.STRING, number=1,)
@@ -326,13 +361,24 @@ class BatchGetDocumentsResponse(proto.Message):
     r"""The streamed response for
     [Firestore.BatchGetDocuments][google.firestore.v1.Firestore.BatchGetDocuments].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         found (google.cloud.firestore_v1.types.Document):
             A document that was requested.
+
+            This field is a member of `oneof`_ ``result``.
         missing (str):
             A document name that was requested but does not exist. In
             the format:
             ``projects/{project_id}/databases/{database_id}/documents/{document_path}``.
+
+            This field is a member of `oneof`_ ``result``.
         transaction (bytes):
             The transaction that was started as part of this request.
             Will only be set in the first response, and only if
@@ -444,6 +490,13 @@ class RunQueryRequest(proto.Message):
     r"""The request for
     [Firestore.RunQuery][google.firestore.v1.Firestore.RunQuery].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         parent (str):
             Required. The parent resource name. In the format:
@@ -455,16 +508,27 @@ class RunQueryRequest(proto.Message):
             ``projects/my-project/databases/my-database/documents/chatrooms/my-chatroom``
         structured_query (google.cloud.firestore_v1.types.StructuredQuery):
             A structured query.
+
+            This field is a member of `oneof`_ ``query_type``.
         transaction (bytes):
-            Reads documents in a transaction.
+            Run the query within an already active
+            transaction.
+            The value here is the opaque transaction ID to
+            execute the query in.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
         new_transaction (google.cloud.firestore_v1.types.TransactionOptions):
             Starts a new transaction and reads the
             documents. Defaults to a read-only transaction.
             The new transaction ID will be returned as the
             first response in the stream.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             Reads documents as they were at the given
             time. This may not be older than 270 seconds.
+
+            This field is a member of `oneof`_ ``consistency_selector``.
     """
 
     parent = proto.Field(proto.STRING, number=1,)
@@ -498,8 +562,8 @@ class RunQueryResponse(proto.Message):
             was set in the request. If set, no other fields will be set
             in this response.
         document (google.cloud.firestore_v1.types.Document):
-            A query result.
-            Not set when reporting partial progress.
+            A query result, not set when reporting
+            partial progress.
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             The time at which the document was read. This may be
             monotonically increasing; in this case, the previous
@@ -525,6 +589,9 @@ class PartitionQueryRequest(proto.Message):
     r"""The request for
     [Firestore.PartitionQuery][google.firestore.v1.Firestore.PartitionQuery].
 
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         parent (str):
             Required. The parent resource name. In the format:
@@ -537,6 +604,8 @@ class PartitionQueryRequest(proto.Message):
             descendants and be ordered by name ascending.
             Other filters, order bys, limits, offsets, and
             start/end cursors are not supported.
+
+            This field is a member of `oneof`_ ``query_type``.
         partition_count (int):
             The desired maximum number of partition
             points. The partitions may be returned across
@@ -718,15 +787,26 @@ class ListenRequest(proto.Message):
     r"""A request for
     [Firestore.Listen][google.firestore.v1.Firestore.Listen]
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         database (str):
             Required. The database name. In the format:
             ``projects/{project_id}/databases/{database_id}``.
         add_target (google.cloud.firestore_v1.types.Target):
             A target to add to this stream.
+
+            This field is a member of `oneof`_ ``target_change``.
         remove_target (int):
             The ID of a target to remove from this
             stream.
+
+            This field is a member of `oneof`_ ``target_change``.
         labels (Sequence[google.cloud.firestore_v1.types.ListenRequest.LabelsEntry]):
             Labels associated with this target change.
     """
@@ -743,17 +823,32 @@ class ListenResponse(proto.Message):
     r"""The response for
     [Firestore.Listen][google.firestore.v1.Firestore.Listen].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         target_change (google.cloud.firestore_v1.types.TargetChange):
             Targets have changed.
+
+            This field is a member of `oneof`_ ``response_type``.
         document_change (google.cloud.firestore_v1.types.DocumentChange):
             A [Document][google.firestore.v1.Document] has changed.
+
+            This field is a member of `oneof`_ ``response_type``.
         document_delete (google.cloud.firestore_v1.types.DocumentDelete):
             A [Document][google.firestore.v1.Document] has been deleted.
+
+            This field is a member of `oneof`_ ``response_type``.
         document_remove (google.cloud.firestore_v1.types.DocumentRemove):
             A [Document][google.firestore.v1.Document] has been removed
             from a target (because it is no longer relevant to that
             target).
+
+            This field is a member of `oneof`_ ``response_type``.
         filter (google.cloud.firestore_v1.types.ExistenceFilter):
             A filter to apply to the set of documents
             previously returned for the given target.
@@ -761,6 +856,8 @@ class ListenResponse(proto.Message):
             Returned when documents may have been removed
             from the given target, but the exact documents
             are unknown.
+
+            This field is a member of `oneof`_ ``response_type``.
     """
 
     target_change = proto.Field(
@@ -782,12 +879,24 @@ class ListenResponse(proto.Message):
 
 class Target(proto.Message):
     r"""A specification of a set of documents to listen to.
+
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         query (google.cloud.firestore_v1.types.Target.QueryTarget):
             A target specified by a query.
+
+            This field is a member of `oneof`_ ``target_type``.
         documents (google.cloud.firestore_v1.types.Target.DocumentsTarget):
             A target specified by a set of document
             names.
+
+            This field is a member of `oneof`_ ``target_type``.
         resume_token (bytes):
             A resume token from a prior
             [TargetChange][google.firestore.v1.TargetChange] for an
@@ -795,15 +904,19 @@ class Target(proto.Message):
 
             Using a resume token with a different target is unsupported
             and may fail.
+
+            This field is a member of `oneof`_ ``resume_type``.
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             Start listening after a specific ``read_time``.
 
             The client must know the state of matching documents at this
             time.
+
+            This field is a member of `oneof`_ ``resume_type``.
         target_id (int):
             The target ID that identifies the target on
-            the stream. Must be a positive number and non-
-            zero.
+            the stream. Must be a positive number and
+            non-zero.
         once (bool):
             If the target should be removed once it is
             current and consistent.
@@ -811,6 +924,7 @@ class Target(proto.Message):
 
     class DocumentsTarget(proto.Message):
         r"""A target specified by a set of documents names.
+
         Attributes:
             documents (Sequence[str]):
                 The names of the documents to retrieve. In the format:
@@ -824,6 +938,9 @@ class Target(proto.Message):
 
     class QueryTarget(proto.Message):
         r"""A target specified by a query.
+
+        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
         Attributes:
             parent (str):
                 The parent resource name. In the format:
@@ -835,6 +952,8 @@ class Target(proto.Message):
                 ``projects/my-project/databases/my-database/documents/chatrooms/my-chatroom``
             structured_query (google.cloud.firestore_v1.types.StructuredQuery):
                 A structured query.
+
+                This field is a member of `oneof`_ ``query_type``.
         """
 
         parent = proto.Field(proto.STRING, number=1,)
@@ -861,6 +980,7 @@ class Target(proto.Message):
 
 class TargetChange(proto.Message):
     r"""Targets being watched have changed.
+
     Attributes:
         target_change_type (google.cloud.firestore_v1.types.TargetChange.TargetChangeType):
             The type of change that occurred.
