@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,14 +36,28 @@ __protobuf__ = proto.module(
 
 class Write(proto.Message):
     r"""A write on a document.
+
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         update (google.cloud.firestore_v1.types.Document):
             A document to write.
+
+            This field is a member of `oneof`_ ``operation``.
         delete (str):
             A document name to delete. In the format:
             ``projects/{project_id}/databases/{database_id}/documents/{document_path}``.
+
+            This field is a member of `oneof`_ ``operation``.
         transform (google.cloud.firestore_v1.types.DocumentTransform):
             Applies a transformation to a document.
+
+            This field is a member of `oneof`_ ``operation``.
         update_mask (google.cloud.firestore_v1.types.DocumentMask):
             The fields to update in this write.
 
@@ -86,6 +100,7 @@ class Write(proto.Message):
 
 class DocumentTransform(proto.Message):
     r"""A transformation of a document.
+
     Attributes:
         document (str):
             The name of the document to transform.
@@ -97,6 +112,14 @@ class DocumentTransform(proto.Message):
 
     class FieldTransform(proto.Message):
         r"""A transformation of a field of the document.
+
+        This message has `oneof`_ fields (mutually exclusive fields).
+        For each oneof, at most one member field can be set at the same time.
+        Setting any member of the oneof automatically clears all other
+        members.
+
+        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
         Attributes:
             field_path (str):
                 The path of the field. See
@@ -104,6 +127,8 @@ class DocumentTransform(proto.Message):
                 the field path syntax reference.
             set_to_server_value (google.cloud.firestore_v1.types.DocumentTransform.FieldTransform.ServerValue):
                 Sets the field to the given server value.
+
+                This field is a member of `oneof`_ ``transform_type``.
             increment (google.cloud.firestore_v1.types.Value):
                 Adds the given value to the field's current
                 value.
@@ -118,6 +143,8 @@ class DocumentTransform(proto.Message):
                 there is positive/negative integer overflow, the
                 field is resolved to the largest magnitude
                 positive/negative integer.
+
+                This field is a member of `oneof`_ ``transform_type``.
             maximum (google.cloud.firestore_v1.types.Value):
                 Sets the field to the maximum of its current
                 value and the given value.
@@ -135,6 +162,8 @@ class DocumentTransform(proto.Message):
                 zero input value is always the stored value.
                 The maximum of any numeric value x and NaN is
                 NaN.
+
+                This field is a member of `oneof`_ ``transform_type``.
             minimum (google.cloud.firestore_v1.types.Value):
                 Sets the field to the minimum of its current
                 value and the given value.
@@ -152,6 +181,8 @@ class DocumentTransform(proto.Message):
                 zero input value is always the stored value.
                 The minimum of any numeric value x and NaN is
                 NaN.
+
+                This field is a member of `oneof`_ ``transform_type``.
             append_missing_elements (google.cloud.firestore_v1.types.ArrayValue):
                 Append the given elements in order if they are not already
                 present in the current field value. If the field is not an
@@ -165,6 +196,8 @@ class DocumentTransform(proto.Message):
                 considered.
 
                 The corresponding transform_result will be the null value.
+
+                This field is a member of `oneof`_ ``transform_type``.
             remove_all_from_array (google.cloud.firestore_v1.types.ArrayValue):
                 Remove all of the given elements from the array in the
                 field. If the field is not an array, or if the field does
@@ -177,6 +210,8 @@ class DocumentTransform(proto.Message):
                 duplicates.
 
                 The corresponding transform_result will be the null value.
+
+                This field is a member of `oneof`_ ``transform_type``.
         """
 
         class ServerValue(proto.Enum):
@@ -221,6 +256,7 @@ class DocumentTransform(proto.Message):
 
 class WriteResult(proto.Message):
     r"""The result of applying a write.
+
     Attributes:
         update_time (google.protobuf.timestamp_pb2.Timestamp):
             The last update time of the document after applying the
@@ -334,6 +370,7 @@ class DocumentRemove(proto.Message):
 
 class ExistenceFilter(proto.Message):
     r"""A digest of all the documents that match a given target.
+
     Attributes:
         target_id (int):
             The target ID to which this filter applies.
