@@ -216,7 +216,6 @@ def test_listen_testprotos(test_proto):  # pragma: NO COVER
     # 'docs' (list of 'google.firestore_v1.Document'),
     # 'changes' (list lof local 'DocChange', and 'read_time' timestamp.
     from google.cloud.firestore_v1 import Client
-    from google.cloud.firestore_v1 import DocumentReference
     from google.cloud.firestore_v1 import DocumentSnapshot
     from google.cloud.firestore_v1 import Watch
     import google.auth.credentials
@@ -226,14 +225,13 @@ def test_listen_testprotos(test_proto):  # pragma: NO COVER
 
     credentials = mock.Mock(spec=google.auth.credentials.Credentials)
     client = Client(project="project", credentials=credentials)
-    modulename = "google.cloud.firestore_v1.watch"
-    with mock.patch("%s.Watch.ResumableBidiRpc" % modulename, DummyRpc):
-        with mock.patch(
-            "%s.Watch.BackgroundConsumer" % modulename, DummyBackgroundConsumer
-        ):
-            with mock.patch(  # conformance data sets WATCH_TARGET_ID to 1
-                "%s.WATCH_TARGET_ID" % modulename, 1
-            ):
+    # conformance data has db string as this
+    db_str = "projects/projectID/databases/(default)"
+    client._database_string_internal = db_str
+    with mock.patch("google.cloud.firestore_v1.watch.ResumableBidiRpc"):
+        with mock.patch("google.cloud.firestore_v1.watch.BackgroundConsumer"):
+            # conformance data sets WATCH_TARGET_ID to 1
+            with mock.patch("google.cloud.firestore_v1.watch.WATCH_TARGET_ID", 1):
                 snapshots = []
 
                 def callback(keys, applied_changes, read_time):
@@ -241,12 +239,7 @@ def test_listen_testprotos(test_proto):  # pragma: NO COVER
 
                 collection = DummyCollection(client=client)
                 query = DummyQuery(parent=collection)
-                watch = Watch.for_query(
-                    query, callback, DocumentSnapshot, DocumentReference
-                )
-                # conformance data has db string as this
-                db_str = "projects/projectID/databases/(default)"
-                watch._firestore._database_string_internal = db_str
+                watch = Watch.for_query(query, callback, DocumentSnapshot)
 
                 wrapped_responses = [
                     firestore.ListenResponse.wrap(proto) for proto in testcase.responses
